package be.iminds.ilabt.jfed.restapi.json;


import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.RFC3339Util;

import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.notNullValue;

public class ExperimentTestCollection {
    private final List<ExperimentBuilder> all;

    public ExperimentTestCollection() throws Exception {
        all = new ArrayList<>();

        ExperimentBuilder exp1 = new ExperimentBuilder();
//        exp1.setName("myExpName");
        exp1.setId(1548l);

        exp1.setStatus(Experiment.Status.STARTING);
        exp1.setAllocationStatus("geni_provisioned");
        exp1.setOperationalStatus("geni_configuring");
        exp1.setJFedExperimentState("AM_WAIT_UNTIL_READY");
//        exp1.setStatusDetails("XXXXXXXXXX");

        exp1.setAm(new URL("http://fls.fed4fire.eu/api2/server/320"));
        exp1.setCreated(new Timestamp(RFC3339Util.rfc3339StringToDate("2015-11-30T10:00:00Z").getTime()));
        exp1.setExpire(new Timestamp(RFC3339Util.rfc3339StringToDate("2015-11-30T16:00:00Z").getTime()));
        exp1.setManifestRspec("<rspec ...>");
        exp1.setRequestRspec(null);
        exp1.setOwner(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+user+user1"));
        exp1.addSharedWith(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+user+user2"));
        exp1.setProject("test");
        exp1.setSliceName("slice1");
        exp1.setSliceUrn(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+slice+slice1"));
        exp1.setSpeakingFor("urn:publicid:IDN+example.com+user+spokenForUsers");
        exp1.setSpeaksForCredential("<SPEAKSFOR CREDENTIAL XML HERE>");

        all.add(exp1);
    }

    public String getFixtureByIndex(int index) {
        return "fixtures/experiment-"+index+".json";
    }

    public Experiment getById(Long id) {
        for (ExperimentBuilder exp : all)
            if (exp.getId() != null && exp.getId().equals(id))
                return exp.create();
        return null;
    }

    public Experiment getByIndex(int index) {
        return all.get(index).create();
    }

    public int getSize() {
        return all.size();
    }

    public void setUri(int i, String s) {
        ExperimentBuilder eb = all.get(i);
        try {
            eb.setUri(new URI(s + "/experiment/" + eb.getId()));
        } catch (URISyntaxException e) {
            throw new RuntimeException("Invalid base URL specified", e);
        }
    }

    public boolean assertSame(Experiment expected, Experiment actual, boolean expectUriInActual) {
        assertThat(expected, is(notNullValue()));
        assertThat(actual, is(notNullValue()));
        assertThat(actual.getAm(), is(equalTo(expected.getAm())));
        assertThat(actual.getCreated(), is(equalTo(expected.getCreated())));
        assertThat(actual.getExpire(), is(equalTo(expected.getExpire())));
        assertThat(actual.getId(), is(equalTo(expected.getId())));
        assertThat(actual.getManifestRspec(), is(equalTo(expected.getManifestRspec())));
//        assertThat(actual.getName(), is(equalTo(expected.getName())));
        assertThat(actual.getOwner(), is(equalTo(expected.getOwner())));
        assertThat(actual.getProject(), is(equalTo(expected.getProject())));
        assertThat(actual.getRequestRspec(), is(equalTo(expected.getRequestRspec())));
        assertThat(actual.getSharedWith(), is(equalTo(expected.getSharedWith())));
        assertThat(actual.getSliceName(), is(equalTo(expected.getSliceName())));
        assertThat(actual.getSliceUrn(), is(equalTo(expected.getSliceUrn())));
        assertThat(actual.getSpeakingFor(), is(equalTo(expected.getSpeakingFor())));
        assertThat(actual.getSpeaksForCredential(), is(equalTo(expected.getSpeaksForCredential())));
        assertThat(actual.getStatus(), is(equalTo(expected.getStatus())));
        assertThat(actual.getStatusDetails(), is(equalTo(expected.getStatusDetails())));

        if (expectUriInActual) {
            assertThat(actual.getUri(), is(notNullValue()));
            if (expected.getUri() != null)
                assertThat(actual.getUri(), is(equalTo(expected.getUri())));
        }
        return true;
    }
}
